Imports System.Reflection
Imports System.CodeDom.Compiler
Imports System.Text

Public Class VBCode

    ' What kind of libraries that'll be allowed
    Public Libraries As List(Of String)

    ' Input
    Public CompilerParamenters As New CompilerParameters

    ' "Return values"
    Public CompilerResults As CompilerResults
    Public CompilerErrors As CompilerErrorCollection

    ' Used to save the current filters we're using
    Public FilterProcess As String = ""
    Public FilterProcessEnd As String = ""
    Public FilterSetPixel As String = ""
    Public FilterEndPixel As String = ""

    ' The class we'll compile to
    Public ClassInstance As Object
    Public Assembly As Assembly

    ' The code that was last compiled
    Public ClassCode As String

    ' The default name of the namespace and class
    Public DefaultNamespace As String = "Script"
    Public DefaultClass As String = "Main"

    Public Sub New()
        ' Just create the class.
    End Sub

    Public Sub New(ByVal aLibraries() As String)

        ' As the above
        InitializeCompiler(aLibraries)

    End Sub

    Public Sub New(ByVal aLibraries() As String, ByVal Code As String)

        ' As the above
        InitializeCompiler(aLibraries)

        ' Compile the given code
        Compile(Code)

    End Sub

    Private Sub InitializeCompiler(ByVal aLibraries() As String)

        Dim sLibrary As String

        ' Create a new list
        Libraries = New List(Of String)

        ' Add the elements to the list
        For Each sLibrary In aLibraries
            Libraries.Add(sLibrary)
        Next

        ' Initialize the compiler
        With CompilerParamenters
            .GenerateInMemory = True
            .MainClass = DefaultNamespace & "." & DefaultClass
            .ReferencedAssemblies.Add("system.dll")
            .ReferencedAssemblies.Add("system.drawing.dll")
            .ReferencedAssemblies.Add("microsoft.visualbasic.dll")
            .CompilerOptions = "/t:library /removeintchecks+ /optimize+"
        End With

    End Sub

    Public Function Run(ByVal Method As String, ByVal ParamArray Paramenters() As Object) As Object

        ' Attempt to run this function
        Return ClassInstance.GetType.GetMethod(Method).Invoke(ClassInstance, Paramenters)

    End Function

    Public Function Compile(ByVal sMethods As String) As Boolean

        Dim oCode As New StringBuilder, sLibrary As String

        ' Firstly, append the libraries (import)
        For Each sLibrary In Libraries
            oCode.AppendLine("Imports " & sLibrary)
        Next

        ' Add the namespace and class, ...
        oCode.AppendLine("Namespace " & DefaultNamespace)
        oCode.AppendLine("Class " & DefaultClass)

        ' ... the code, ...
        oCode.AppendLine(sMethods)

        ' ... and the end of the class and namespace.
        oCode.AppendLine("End Class")
        oCode.AppendLine("End Namespace")

        ' Return the result of the function below
        If Compile(oCode) Then
            Compile = True
        End If

    End Function

    Public Function Compile(ByVal oModuleCode As StringBuilder) As Boolean

        Dim oCodeProvider As New VBCodeProvider

        ' Attempt to compile the code
        CompilerResults = oCodeProvider.CompileAssemblyFromSource(CompilerParamenters, _
         oModuleCode.ToString)

        ' Save the list of errors
        CompilerErrors = CompilerResults.Errors

        ' Return true if the result is free of errors
        If CompilerErrors.Count = 0 Then

            ' Reference the assembly
            Me.Assembly = CompilerResults.CompiledAssembly

            ' Create an instance of the class
            Me.ClassInstance = Me.Assembly.CreateInstance(CompilerParamenters.MainClass)

            ' The compilation was successful
            Compile = True

        Else

            ' Inform that something have gone wrong
            Throw New Exception(ConvertToText(CompilerErrors))

        End If

    End Function

    Private Function ConvertToText(ByVal List As CompilerErrorCollection) As String

        Dim oError As CompilerError, oResult As New StringBuilder

        ' Go through all the errors
        For Each oError In List
            ' Add the line to the string
            oResult.AppendLine(oError.ErrorNumber & " in line " & oError.Line & ", column " & _
             oError.Column & ": " & oError.ErrorText)
        Next

        ' Return the generated text
        Return oResult.ToString

    End Function

End Class