Imports System.Windows.Forms
Imports System.Xml.Serialization
Imports System.IO

Public Class frmFilters

    ' Contains all the registered filters
    Public Filters As New List(Of Filter)

    Private Sub cmdOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOk.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub

    Private Sub cmdCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdCancel.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Load all filters.
        LoadFilters(IO.Path.Combine(IO.Path.GetDirectoryName(Application.ExecutablePath), _
         "Data" & Path.DirectorySeparatorChar & "Filters.xml"))

        ' Load comboboxes
        UpdateLists()

    End Sub

    ' Updates the content of the lists
    Public Sub UpdateLists()

        Dim Filter As Filter

        ' Clear lists
        cmdProcess.Items.Clear()
        cmdProcessEnd.Items.Clear()
        cmdPixelSet.Items.Clear()
        cmdPixelEnd.Items.Clear()

        ' Go through all filters, ...
        For Each Filter In Filters

            ' ... adding them to the appropriate list
            Select Case Filter.FilterType
                Case Fractal.Filter.FilterTypes.Process
                    cmdProcess.Items.Add(Filter.Code)
                Case Fractal.Filter.FilterTypes.ProcessEnd
                    cmdProcessEnd.Items.Add(Filter.Code)
                Case Fractal.Filter.FilterTypes.PixelSet
                    cmdPixelSet.Items.Add(Filter.Code)
                Case Fractal.Filter.FilterTypes.PixelEnd
                    cmdPixelEnd.Items.Add(Filter.Code)
            End Select

        Next

    End Sub

    ' Used to load all filters
    Public Function LoadFilters(ByVal FilePath As String) As Boolean

        Dim File As FileStream, Formatter As XmlSerializer

        Try

            ' Firstly, open the file
            File = New FileStream(FilePath, IO.FileMode.Open)
            Formatter = New XmlSerializer(Filters.GetType)

            ' Retrieve the list
            Filters = Formatter.Deserialize(File)

            ' Everything went OK
            LoadFilters = True

        Catch ex As Exception
            MessageBox.Show("Cannot load filters: " & Environment.NewLine & ex.Message, _
             "Error", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        Finally

            ' Clean up
            If Not (File Is Nothing) Then
                File.Dispose()
            End If

        End Try

    End Function

    ' Used to load all filters
    Public Sub SaveFilters(ByVal FilePath As String)

        Dim File As FileStream, Writer As StreamWriter, Formatter As XmlSerializer

        Try

            ' Firstly, open the file
            File = New FileStream(FilePath, IO.FileMode.Create)
            Writer = New StreamWriter(File)
            Formatter = New XmlSerializer(Filters.GetType)

            ' Retrieve the list
            Formatter.Serialize(Writer, Filters)

        Catch ex As Exception
            MessageBox.Show("Cannot save filters: " & Environment.NewLine & ex.Message, "Error", _
             MessageBoxButtons.OK, MessageBoxIcon.Warning)
        Finally

            ' Clean up
            If Not (Writer Is Nothing) Then
                Writer.Close()
            End If

        End Try

    End Sub

End Class

' Represents a filter
<System.Serializable()> Public Class Filter

    ' Contains the different filter types
    Public Enum FilterTypes
        Process = 1
        ProcessEnd = 2
        PixelSet = 3
        PixelEnd = 4
    End Enum

    ' Data about this filter
    Public FilterType As FilterTypes
    Public Code As String

    Public Sub New()
        ' Simply let the filter get created.
    End Sub

    ' Used to construct these filters
    Public Sub New(ByVal Code As String, ByVal FilterType As FilterTypes)
        ' Set the properties
        With Me
            .Code = Code
            .FilterType = FilterType
        End With
    End Sub

End Class