Public Class frmProperties

    ' Sets/returns the array this data grid will be displaying
    Public DataObjects As List(Of VBCode)

    ' Whether or not to ignore the FormClosing-event
    Public AllowClose As Boolean

    ' Contains the property names
    Private PropertyNames As List(Of String)
    Private IgnoreRegresion As Boolean

    Public ReadOnly Property SelectedItem() As VBCode
        Get
            Return DataObjects(lstObjects.SelectedIndex)
        End Get
    End Property

    Private ReadOnly Property PropertiesCount() As Integer
        Get
            ' Go through all the field, ...
            Return DataObjects(lstObjects.SelectedIndex).ClassInstance.GetType.GetFields.Length
        End Get
    End Property

    Public Sub UpdateList()

        Dim Tell As Integer

        ' Used to avoid regresion
        If IgnoreRegresion Then
            Exit Sub
        Else
            IgnoreRegresion = True
        End If

        ' Clear list
        lstObjects.Items.Clear()

        ' Add all the objects to the list
        For Tell = 0 To DataObjects.Count - 1
            lstObjects.Items.Add(ReadValue(Tell, "Name"))
        Next

        ' Select the first element
        lstObjects.SelectedIndex = 0

        ' We're done
        IgnoreRegresion = False

    End Sub

    Public Sub UpdateRows()

        Dim oProperty As Reflection.MemberInfo

        ' Updates the amount of rows in the list
        If PropertiesCount > 0 Then

            ' Update the property list
            PropertyNames = New List(Of String)

            ' Set the amount of rows
            dataProperties.RowCount = PropertiesCount + 1

            ' Go through all the properties in this object, ...
            For Each oProperty In SelectedItem.ClassInstance.GetType.GetMembers
                ' ... adding all properties.
                If oProperty.MemberType = Reflection.MemberTypes.Field Then
                    PropertyNames.Add(oProperty.Name)
                End If
            Next

            ' Sort the properties alphabetically
            PropertyNames.Sort()

        Else
            ' Avoids an error
            dataProperties.RowCount = 1
        End If

    End Sub

    Private Sub dataProperties_CellClick(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellEventArgs) Handles dataProperties.CellClick

        ' See if this is the filters-cell
        If e.ColumnIndex = 1 And e.RowIndex = dataProperties.RowCount - 1 Then

            ' Initialize the filters dialog
            With SelectedItem
                frmFilters.cmdProcess.Text = .FilterProcess
                frmFilters.cmdProcessEnd.Text = .FilterProcessEnd
                frmFilters.cmdPixelSet.Text = .FilterSetPixel
                frmFilters.cmdPixelEnd.Text = .FilterEndPixel
            End With

            ' If the user accepted the changes, ...
            If frmFilters.ShowDialog = Windows.Forms.DialogResult.OK Then

                With SelectedItem

                    ' ... apply them.
                    .FilterProcess = frmFilters.cmdProcess.Text
                    .FilterProcessEnd = frmFilters.cmdProcessEnd.Text
                    .FilterSetPixel = frmFilters.cmdPixelSet.Text
                    .FilterEndPixel = frmFilters.cmdPixelEnd.Text

                    ' Update the compilation
                    frmMain.EditClass(SelectedItem, .ClassCode)

                End With

            End If

        End If

    End Sub

    Private Sub dataProperties_CellValueNeeded(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellValueEventArgs) Handles dataProperties.CellValueNeeded

        Try

            ' See if this is the last row
            If e.RowIndex = dataProperties.RowCount - 1 Then

                ' If so, make this the filters-cell.
                If e.ColumnIndex = 0 Then
                    e.Value = "Filters"
                End If

                ' We're done
                Exit Sub

            End If

            ' The value is retrieved differently depending on the column
            Select Case e.ColumnIndex
                Case 0
                    ' Just use the stored array.
                    e.Value = PropertyNames(e.RowIndex)

                Case 1

                    ' Otherwise, invoke this property to retrieve its information.
                    e.Value = ReadValue(PropertyNames(e.RowIndex))

            End Select

        Catch ex As Exception
            e.Value = ""
        End Try

    End Sub

    Private Function ReadValue(ByVal Index As Integer, ByVal Name As String) As String

        Try
            ' Return the property's value
            With DataObjects(Index).ClassInstance.GetType.GetField(Name)
                Return .GetValue(DataObjects(Index).ClassInstance)
            End With
        Catch ex As Exception
            Return ""
        End Try

    End Function

    Private Function ReadValue(ByVal Name As String) As String

        ' Use the above function
        Return ReadValue(lstObjects.SelectedIndex, Name)

    End Function

    Private Sub dataProperties_CellValuePushed(ByVal sender As Object, ByVal e As System.Windows.Forms.DataGridViewCellValueEventArgs) Handles dataProperties.CellValuePushed

        Dim oField As Reflection.FieldInfo, vVar As Object

        ' Set the new value
        If e.ColumnIndex = 1 And e.RowIndex < dataProperties.RowCount - 1 Then

            ' Get the field
            oField = SelectedItem.ClassInstance.GetType.GetField(PropertyNames(e.RowIndex))

            Try

                ' Simply annoying that this is the only way to accomplish this. :(
                Select Case oField.FieldType.Name
                    Case "Decimal" : vVar = Decimal.Parse(e.Value)
                    Case "Boolean" : vVar = Boolean.Parse(e.Value)
                    Case "Double" : vVar = Double.Parse(e.Value)
                    Case "Single" : vVar = Single.Parse(e.Value)
                    Case "Long", "Int64" : vVar = Long.Parse(e.Value)
                    Case "Integer", "Int32" : vVar = Integer.Parse(e.Value)
                    Case "Short", "Int16" : vVar = Short.Parse(e.Value)
                    Case "Byte" : vVar = Byte.Parse(e.Value)
                    Case Else : vVar = e.Value
                End Select

                ' Set the value
                oField.SetValue(SelectedItem.ClassInstance, vVar)

            Catch ex As Exception
                MessageBox.Show("Error, cannot set field: " & Environment.NewLine & ex.Message, _
                 "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try

        End If

    End Sub

    Private Sub frmProperties_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        ' Avoid the form being closed completely
        If Not AllowClose Then
            e.Cancel = True
            Me.Hide()
        End If

    End Sub

    Private Sub frmProperties_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.VisibleChanged
        ' Set the checkbox
        frmMain.mnuProperties.Checked = Me.Visible
    End Sub

    Private Sub lstObjects_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstObjects.SelectedIndexChanged

        ' Update the form
        UpdateRows()

        ' Redraw everything
        dataProperties.Refresh()

    End Sub

End Class